import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';

abstract class BaseWidget extends GetWidget<BaseController>{
  BaseWidget(BaseController baseController) {
    Get.put(baseController);
  }
  S getController<S>() {
    return this.controller as S;
  }

  @override
  Widget build(BuildContext context) {
    return this.getWidget(context);
  }

  Widget getWidget(BuildContext context) {
    return GetBuilder(
      init: this.controller,
      initState: (_) async {
        this.controller.initStateGetX();
      },
      builder: (BaseController controller) {
        printInfo(info: 'GetWidget start');
        Get.find<BaseController>()?.rxData?.value;
        var cachedData = this.controller.getData();
        if (cachedData.length == 0) {
          return new FutureBuilder(
            future: this.controller.loadData(),
            builder: (context,
                AsyncSnapshot<Map<String, List<Map<String, Object>>>>
                snapshot) {
              printInfo(info: 'GetWidget rebuild new data');

              if (snapshot.connectionState == ConnectionState.waiting) {
                return this.getSkelectonView(context);
              } else if (snapshot.data == null || snapshot.hasError) {
                return this.getErrorView(snapshot);
              } else if (snapshot.data.length == 0) {
                return this.getEmptyView();
              }
              this.controller.setData(snapshot.data);
              return this.getMainView(context, snapshot.data, controller);
            },
          );
        } else {
          printInfo(info: 'GetWidget rebuild old data');
          return this.getMainView(context, cachedData,controller );
        }
      },
    );
  }

  Widget getSkelectonView(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(left: 10),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            children: [
              Expanded(
                flex: 6,
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Container(
                    height: 150,
                    decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100]),
                  ),
                ),
              ),
              Expanded(
                flex: 3,
                child: Column(
                  children: [
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(10),
                            color: Colors.grey[100]),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(10),
                            color: Colors.grey[100]),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
          Padding(
            padding:
            const EdgeInsets.only(top: 8, right: 8, left: 8, bottom: 8),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  color: Colors.grey[100]),
            ),
          ),
          Padding(
            padding:
            const EdgeInsets.only(top: 8, right: 20, left: 8, bottom: 8),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width / 1.5,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  color: Colors.grey[100]),
            ),
          )
        ],
      ),
    );
  }

  Widget getErrorView(
      AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot) {
    return Center(
      child:
      Image.asset("assets/images/404-error.png", height: 100, width: 100),
    );
  }

  Widget getEmptyView() {
    return Center(
      child:
      Image.asset("assets/images/emptyFolder.png", height: 100, width: 100),
    );
  }

  Widget getMainView(
      BuildContext context, Map<String, List<Map<String, Object>>> data, BaseController controller);

}
